#include <iostream> // biblioteka do obslugi standardowego wejscia wyjscia

// struktura reprezentujaca liczbe zespolona
// struktury w C++ sa de facto klasami o pewnych specyficznych wlasciwosciach / wiecej na wykladzie
struct complex
{
	double r_;
	double i_;
};

// dodawanie w stylu jezyka C
complex add(const complex a, const complex b)
{
	return { a.r_ + b.r_, a.i_ + b.i_ };
}

// odejmowanie w stylu jezyka C
complex substract(const complex a, const complex b)
{
	return { a.r_ - b.r_, a.i_ - b.i_ };
}

// mnozenie w stylu jezyka C
complex multiply(const complex a, const complex b)
{
	return { a.r_ * b.r_ - a.i_ * b.i_, 
		a.r_ * b.i_ + a.i_ * b.r_ };
}

// dzielenie w stylu jezyka C
complex divide(const complex a, const complex b)
{
	auto mod2b = b.r_ *b.r_ + b.i_ * b.i_;
	if ( 0 == mod2b)
		return { 0,0 };

	return { (a.r_ * b.r_ + a.i_ * b.i_) / mod2b,  (a.i_ * b.r_ - a.r_ * b.i_) / mod2b };
}

// dodawanie w stylu jezyka C++
complex operator+(const complex& a, const complex& b)
{
	return add(a, b); // po co powtarzac kod, skoro jest juz napisany!!! 
}

// odejmowanie w stylu jezyka C++
complex operator-(const complex& a, const complex& b)
{
	return substract(a, b);
}

// mnozenie w stylu jezyka C++
complex operator*(const complex& a, const complex& b)
{
	return multiply(a, b);
}

// dodawanie w stylu jezyka C++
complex operator/(const complex& a, const complex& b)
{
	return divide(a, b);
}

// wyswietlenie liczby zespolonej jako funkcja
void print(const complex a)
{
	std::cout << a.r_; // obsluga wyswietlania w stylu C++ / wiecej mozna dowiedziec sie na wykladzie lub https://en.cppreference.com/w/cpp/io/cout
	if (a.i_ >= 0)
		std::cout << " + " << a.i_<< "j";
	else
		std::cout << " - " << -a.i_ << "j";;
}

// wyswietlanie liczby zespolonej w stylu C++ 
std::ostream& operator<<(std::ostream& os, const complex a)
{
	os << a.r_;
	if (a.i_ >= 0)
		os << " + " << a.i_ << "j";
	else
		os << " - " << -a.i_ << "j";
	return os;
}

// operacje na liczba zespolonych
// podniesienie liczby zespolonej do potegi bedacej liczba naturalna / uwaga : algorytm naiwny wykorzystujacy rekurencje - istnieja algorytmy znacznie bardziej efektwyne.
complex power(const complex a, unsigned int i)
{
	if (0 == i)
		return { 1,0 };
	else
		return a * power(a, i - 1);
}

int power(int a, unsigned int i)
{
	if (0 == i)
		return 1;
	else
		return a * power(a, i - 1);
}

long long factorial(long long i)
{
	if (0 == i)
		return 1;
	else
		return (factorial(i - 1)*i);
}

// obliczenie wartosci funkcji sinus arumentu zespolonego, wykorzystujac rozwiniecie w szereg Taylora. 
complex sin(const complex a)
{
	unsigned int N = 20; // ilosc branych pod uwage wyrazow szeregu Taylora

	complex r = { 0,0 }; // zwracany wynik, ktory bedzie aktualizowany

	for (unsigned int i = 0; i < N; ++i) // petla 
	{
		complex k = { double(power(-1, i)) / double(factorial(2 * i + 1)) , 0 };
		r = r + power(a, 2 * i + 1) * k;
	}
	return r;
}


int main(void)
{
	// stworzenie obiektow reprezentujacych liczby zespolone; 
	complex a = { 1,2 };
	complex b = { 1,-2 };

	// przyklad wykorzystania stworzonego kodu w stylu C
	print(a);
	std::cout << std::endl; // przejscie do nowej linii;
	print(b);
	std::cout << std::endl;

	print(add(a, b));
	std::cout << std::endl;

	print(substract(a, b));
	std::cout << std::endl;

	print(multiply(a, b));
	std::cout << std::endl;

	print(divide(a, b));
	std::cout << std::endl;


	// przyklad wykorzystania stworzonego kodu w stylu C++
	std::cout << "a = " << a << ", b = " << b << std::endl;	
	std::cout << "a + b = " << a + b << std::endl;
	std::cout << "a - b = " << a - b << std::endl;
	std::cout << "a * b = " << a * b << std::endl;
	std::cout << "a / b = " << a / b << std::endl;

	std::cout << "a^10 = "<< power(a, 10) << std::endl;
	
	std::cout << "sin(a) = " << sin(a) << std::endl;
	return 0;
}